# -*- coding: utf-8 -*-

# Copyright 2006-2012 Borislav Rizov <bobyATdclDOTbasDOTbg>, Department of Computational Linguistic, Institute for Bulgarian Language – Bulgarian Academy of Sciences
# 
# This file is part of Hydra.
# 
# Hydra is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hydra is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Hydra.  If not, see <http://www.gnu.org/licenses/>.

import types
from descriptor import  *

class Converter:
    def var(self, variable):
        pass
    def neg(self, form):
        pass
    def conj(self, form1, form2):
        pass
    def disj(self, form1, form2):
        pass
    def impl(self, form1, form2):
        pass
    def equ(self, form1, form2):
        pass
    def romb(self, form):
        pass		
    def squ(self, form):
        pass	
        
class SimpleConverter(Converter):
    def var(self, variable):
        return 'var(' + variable + ')'
    def neg(self, form):
        return '!' + form
    def conj(self, form1, form2):
        return 'conj(' + form1 + ', ' + form2 + ')' 
    def disj(self, form1, form2):
        return 'disj(' + form1 + ', ' + form2 + ')' 
    def impl(self, form1, form2):
        return 'impl(' + form1 + ', ' + form2 + ')' 
    def equ(self, form1, form2):
        return 'equ(' + form1 + ', ' + form2 + ')' 
    def romb(self, rel, form):
        return 'romb(' + rel + ', ' + form + ')' 
    def squ(self, rel, form):
        return 'squ(' + rel + ', ' + form + ')' 
    def func(self, fun, const):
        return 'func(' + fun + ', ' + const + ')'
    def rel(self, rel):
        return 'rel(' + rel + ')'
    def revrel(self, rel):
        return 'revrel(' + rel + ')'
    def const(self, c):
        return 'const(' + c + ')'
class SQLConverter(Converter):
    v = 'Val'
    rels = RelId.table
    world = 'id'
    pr1 = 'id1'
    pr2 = 'id2'
    atom = 'atom'
    lits = LiteralId.table
    main = SynsetId.table
    notes = NoteId.table
    i = 0
    def frame(self):
        return '(select ' + self.world + ' from ' + self.main + ' union select ' + self.world + ' from ' + self.lits + ' union select ' + self.world + ' from ' + self.notes + ')'
    def const(self, c):
        return '(select %s as id)' %c;
    def var(self, p):
        if p in ('$s', '$q'):
            table = self.main
        elif p in ('$l', '$p'):
            table = self.lits
        elif p in ('$n', '$r'):
            table = self.notes
        else:
            raise NameError, "invalid variable"
        return '(select ' + self.world + ' from ' + table + ')'
    def neg(self, form):
        Fi = 'F' + `self.i`
        Qi = 'Q' + `self.i`
        res1 = '(select ' + Fi + '.' + self.world + ' from ' + self.frame() + ' ' +  Fi + ' where not exists ' + '(select 1 from ' + form + 'as ' + Qi + ' where ' + Qi + '.id = ' + Fi + '.id)' + ')'
        self.i += 1
        return res1
    def conj(self, form1, form2):
        i = `self.i`
        j = `self.i + 1`
        res =  ' (select Q' + i + '.' + self.world +\
            ' from (' + form1 + ') as Q' + i +\
            ' inner join (' + form2 + ') as Q' + j +\
            ' on Q' + i + '.' + self.world + '=Q' + j + '.' + self.world + ')'
        self.i += 2
        return res
    def disj(self, form1, form2):
# 		i = `self.i`
# 		self.i += 1
        return "%s union %s" % (form1, form2)
    def impl(self, form1, form2):
        return self.disj(self.neg(form1), form2)
    def equ(self, form1, form2):
        return self.conj(self.impl(form1, form2), self.impl(form2, form1))
    def relid(self, rel):
        return "(select id from Relations where name = '%s')" % rel
    def rel(self, rel):
        from wordnet import wn
        rel = rel.lower()
        relation = wn.relations.get(rel,  None)
        if not relation: relation = wn.sys_relations[rel]
        return  not relation.reversed,  unicode(relation.info[0])
    def revrel(self, rel):
        res = self.rel(rel)
        return not res[0],  res[1]

    def romb(self, rel, form):
        i = `self.i`
        if(rel[0]):
            pr1 = self.pr1
            pr2 = self.pr2
        else:
            pr2 = self.pr1
            pr1 = self.pr2
# 		print rel
        res = ' (select distinct R' + i + '.' + pr1 + ' as ' + self.world + ' from ' + form + ' as Q' + i + \
            ' inner join ' + self.rels + ' as R' + i + \
            ' on R' + i + '.' + pr2 + '= Q' + i + '.' + self.world + ' and rel = ' + rel[1] + ')'
        self.i += 1
        return res
    def major_romb_fraction(self,  rel,  p,  q,  form):
        i = `self.i`
        if(rel[0]):
            pr1 = self.pr1
            pr2 = self.pr2
        else:
            pr2 = self.pr1
            pr1 = self.pr2
        t1 = 'select %s, count(%s) as c%s from Rel inner join %s as F on Rel.%s = F.%s and rel=%s group by %s' \
            % (pr1, pr1, i, form,  pr2,  self.world, rel[1],   pr1)
        t2 = 'select %s, count(%s) as d%s from Rel group by %s' \
                % (pr1,  pr1,  i,  pr1)
        res = '(select T1.%s as %s from (%s) as T1 inner join (%s) as T2 on T1.%s = T2.%s where c%s*%s > d%s*%s)' \
                %(pr1, self.world, t1, t2, pr1, pr1, i, q, i,  p)
        self.i += 1
        hristo = """
        select T1.id2 as id 
from (
        select id2, count(id2) as c11
        from Rel
        inner join Main on Rel.id1 = Main.id
        inner join Relations on Rel.id1 = Relations.id
        where name = 'holo_part'
        group by id2
    ) as T1
inner join (
                select id2, count(id2) as d11 
                from Rel group by id2
            ) as T2
    on T1.id2 = T2.id2 
where c11*4 > d11*3
        """
#		res = "select T1.id2 as id from (	select id2, count(id2) as c11		from Rel		inner join Main on Rel.id1 = Main.id		inner join Relations on Rel.id1 = Relations.id		where name = 'holo_part'		group by id2	) as T1inner join (				select id2, count(id2) as d11 				from Rel group by id2			) as T2	on T1.id2 = T2.id2 where c11*4 > d11*3"
#        print res
        return res
    def major_romb_exact(self,  rel,  number,  form):
        i = `self.i`
        if(rel[0]):
            pr1 = self.pr1
            pr2 = self.pr2
        else:
            pr2 = self.pr1
            pr1 = self.pr2
        res = 'select %s as %s from Rel inner join %s as F on Rel.%s = F.%s and rel=%s group by %s having count(%s) = %s' \
            % (pr1, self.world,  form,  pr2,  self.world, rel[1],   pr1,  pr1,  number)
#        print res
#		r = 'select %s, count(%s) as c%s from Rel inner join %s as F on Rel.%s = F.%s and rel=%s group by %s' \
#			% (pr1, pr1, i, form,  pr2,  self.world, rel[1],   pr1)
#		res = '(select %s as %s from (%s) as D%s where c%s = %s)' % (pr1,  self.world, r, i, i, number)
        return res
    def major_romb(self,  rel,  number,  form):
        i = `self.i`
        if(rel[0]):
            pr1 = self.pr1
            pr2 = self.pr2
        else:
            pr2 = self.pr1
            pr1 = self.pr2
        r = 'select %s, count(%s) as c%s from Rel inner join %s as F on Rel.%s = F.%s and rel=%s group by %s' \
            % (pr1, pr1, i, form,  pr2,  self.world, rel[1],   pr1)
        res = '(select %s as %s from (%s) as D%s where c%s > %s)' % (pr1,  self.world, r, i, i, number)
#        print res
        return res
    def major_squ_fraction(self,  rel,  p,  q,  form):
        return self.neg(self.major_romb_fraction(rel, p, q, self.neg(form)))
    def major_squ(self,  rel,  number,  form):
        return self.neg(self.major_romb(rel, number,  self.neg(form)))
    def squ(self, rel, form):
        return self.neg(self.romb(rel, self.neg(form)))
    def func(self, fun, const):
       
        if fun in SynsetId.fields:
            descriptor = SynsetId
        elif fun == 'lid' or fun in LiteralId.fields:
            descriptor = LiteralId
            if fun == 'lid': fun = 'id'
        elif fun == 'nid' or fun in NoteId.fields:
            descriptor = NoteId
            if fun == 'nid': fun = 'id'
        else:
            raise NameError, "unknown constant"

        table = descriptor.table
        if fun in descriptor.foreign:
            const = self.foreign(descriptor, fun, const)
        
        if type(const) is types.UnicodeType and const[0] == '#':
            return '(select ' + self.world + ' from ' + table + ' where ' + fun + ' regexp( ' + const[1:] + '))'
            #return '(select ' + self.world + ' from ' + table + ' where ' + fun + ' like ' + const[1:] + ')'
        return '(select ' + self.world + ' from ' + table + ' where ' + fun + '=' + const + ')'
 
    def foreign(self, descriptor, fun, const):
        f_descriptor = descriptor.foreign[fun]
        field = f_descriptor.fields[1]
        return '(select id from %s where %s=%s)' % (f_descriptor.table, field, const)
        
