# -*- coding: utf-8 -*-

# Copyright 2006-2012 Borislav Rizov <bobyATdclDOTbasDOTbg>, Department of Computational Linguistic, Institute for Bulgarian Language – Bulgarian Academy of Sciences
# 
# This file is part of Hydra.
# 
# Hydra is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hydra is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Hydra.  If not, see <http://www.gnu.org/licenses/>.

class TableType(type):
    def __new__(cls, name, bases, attrs):
        items = attrs.get('fields', ())
        foreign = attrs.get('foreign', {})
        newattrs = attrs.copy()
        for i, x in enumerate(items):
            newattrs[x] = i
        newattrs['foreign'] = foreign
        return super(TableType, cls).__new__(cls, name, bases, newattrs)
    
    def __getitem__(self, name):
        return getattr(self, name)


class Table(object):
    __metaclass__ = TableType

class POSId(Table):
    table = "POS"
    fields = ('id', 'name')

class DomainId(Table):
    table = "Domain"
    fields = ('id', 'name')

class SemClassId(Table):
    table = 'SemClass'
    fields = ('id', 'name', 'code')

class LangId(Table):
    table = "Lang"
    fields = ('id', 'name', 'synset_id')

class SynsetId(Table):
    table = 'Synset'
    fields = ('id', 'ili', 'pos', 'definition', 'stamp', 'bcs', 'lang',  'frequency', 'domain', 'sem_class')
    foreign = {'pos':  POSId, 'lang': LangId, 'domain': DomainId, 'sem_class': SemClassId}

class LiteralId(Table):
    table = 'Literal'
    fields = ('id', 'sense', 'word', 'lemma')

class NoteId(Table):
    table = 'Note'
    fields = ('id', 'note')

class LockId(Table):
    table = 'Locks'
    fields = ('id', 'stamp', 'user', 'value')

class RelationId(Table):
    table = 'Relation'
    fields = ('id', 'name', 'rname', 'id1', 'id2', 'chars')
    rnames = dict(zip(['id', 'rname', 'name', 'id2', 'id1', 'chars'], range(len(fields))))

class RelId(Table):
    table = 'Rel'
    fields = ('id1', 'id2', 'rel')

if __name__ == '__main__':
    print SynsetId.pos, SynsetId['pos']
