# -*- coding: utf-8 -*-

# Copyright 2006-2012 Borislav Rizov <bobyATdclDOTbasDOTbg>, Department of Computational Linguistic, Institute for Bulgarian Language – Bulgarian Academy of Sciences
# 
# This file is part of Hydra.
# 
# Hydra is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hydra is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Hydra.  If not, see <http://www.gnu.org/licenses/>.

from Tix import *
from wordnet import *
from wn_commands import *
##class HolderState:
##	Main, Search = range(2)
##
class ShowFrameCommand(Command):
	def __init__(self, h, frame, previous):
		Command.__init__(self, h)
		self.previous = previous
		self.frame = frame
	def execute(self):
		self.frame.show(self.previous)
	def unexecute(self):
		self.frame.hide()
		#print "unexecute ShowFrameCommand"

class HideFrameCommand(Command):
	def __init__(self, h, frame):
		Command.__init__(self, h)
		self.frame = frame
	def execute(self):
		self.frame.hide()
	def unexecute(self):
		self.frame.show(self.frame.previous)
		#print "unexecute HideFrameCommand"

class RedrawEditorCommand(Command):
	def __init__(self, h, editor):
		Command.__init__(self, h)
		self.editor = editor
	def execute(self):
		self.editor.redraw()
	def unexecute(self):
		self.editor.redraw()
		#print "unexecute RedrawEditorCommand"

class SelectChildLingCommand(Command):
	def __init__(self, h, editor, storage):
		Command.__init__(self, h)
		self.editor = editor
		self.storage = storage
	def execute(self):
		self.new_ling = self.storage.get()
		path = self.editor.find_child(self.new_ling)
		self.editor.select(path)
		#print "selected child"
	def unexecute(self):		
		self.editor.select_parent()

class SelectParentLingCommand(Command):
	def __init__(self, h, editor):
		Command.__init__(self, h)
		self.editor = editor
	def execute(self):
		self.ling = self.editor.ling
		self.editor.select_parent()
	def unexecute(self):
		path = self.editor.find_child(self.ling)
		self.editor.select(path)


class SingleVisible:
	def show(self):
		self.previous = self.parent.box
		self.parent.set_box(self)
	def hide(self):
		self.parent.set_box(self.previous)

class EditFrame(Frame):
	def create_widgets(self):
		self.edit_button = Button(self, text = "Edit", underline = 0, command = self.edit, state = 'disabled')
		self.edit_button.pack()
		
		self.new_button = Button(self, text = "New synset", underline = 0, command = self.new_synset)
		self.new_button.pack()
		
		self.delete_button = Button(self, text = "Delete synset", underline = 0, state = 'disabled', command = self.delete_synset)
		self.delete_button.pack()
	def __init__(self, dict, parent):
		Frame.__init__(self, parent)
		self.parent = parent
		self.dict = dict
		self.create_widgets()
	def enable(self):
		self.edit_button['state'] = 'normal'
		self.delete_button['state'] = 'normal'
	def disable(self):
		self.edit_button['state'] = 'disabled'
		self.delete_button['state'] = 'disabled'
	
	def new_synset(self):
		self.dict.new_synset()
	def delete_synset(self):
		if askokcancel("Delete synset", "Are you sure?"):
			self.dict.delete_synset()
	def edit(self):
		self.dict.edit_ling()
	#def create_clone_menu(self, dict):
		#dict.ling_view
	#def create_popup(self, dict):
		#popup = PopupMenu(self.holder.top, title = "Clone")
		#popup.menu["postcommand"] = lambda: self.create_submenu(popup, dict)
		#popup.bind_widget(dict.ling_view)
		#popup.bind_widget(dict.ling_view.box)
	def create_submenu(self, popup, dict):
		menu = popup.menu
		menu.delete(0, len(self.holder.dicts))
		for adict in self.holder.dicts:
			if adict != dict and adict.ling:
				menu.add_command(label = adict.name, command = lambda: dict.ling_view.clone(adict.ling))
	def refresh(self, ling):
		if ling:
			self.enable()
		else:
			self.disable()
	def hide(self):
		pass
class SaveFrame(Frame, SingleVisible):
	def create_widgets(self):
		# RelFrame
		self.top_frame = Frame(self)
		self.top_frame.pack()
		
		self.bottom_frame = Frame(self)
		self.rel_combo = ComboBox(self.bottom_frame, label = "Relation:", options='listbox.height -1 label.anchor e entry.width 16')
		self.rel_combo.pack(side = 'left')
		
		self.add_rel_button = Button(self.bottom_frame, text = "Add", command = lambda:self.add_rel(self.rel_combo.entry.get()))
		self.add_rel_button.pack(side = 'left')
		
		self.fill_relations()
		size = self.rel_combo.slistbox.listbox.size()
		if size:
			self.bottom_frame.pack()
			self.rel_combo.pick(0)
		
		self.bottom_frame = Frame(self, relief = SUNKEN)
		self.save_button = Button(self.bottom_frame, text = "Save", underline = 0, command = self.save)
		self.save_button.pack(side = 'left')
		
		self.cancel_button = Button(self.bottom_frame, text = "Cancel", underline = 0, command = self.cancel)
		self.cancel_button.pack(side = 'left')
		self.bottom_frame.pack(side = TOP)
	
	def __init__(self, dict, parent, command, LingClass):
		Frame.__init__(self, parent)
		self.parent = parent
		self.dict = dict
		self.command = command
		self.LingClass = LingClass
		self.create_widgets()
		
	def fill_relations(self):
		for name, rel in wn.relations.iteritems():
			if self.LingClass is rel['id1']:
				if rel.is_(RelChars.Explicit):
					b = Button(self.top_frame, text = name, command = lambda r = name: self.add_rel(r))
					b.pack(side = 'left')
				else:
					self.rel_combo.append_history(name)
	def save(self):
		parent = self.parent_ling()
		self.dict.save_ling(parent)
	def cancel(self):
		self.command.undo()
	def parent_ling(self):
##		print self.command
		if hasattr(self.command, 'sling'):
			return self.command.sling
		return None
	def add_creational_rel(self, rel):
		self.dict.add_creational_rel(rel)
	def add_rel(self, name):
		#print "add_rel:", name
		rel = wn.relations[name]
		if rel.is_(RelChars.NewCreational):
			self.add_creational_rel(rel)
		else:
			self.show_searcher(rel)
	def show_searcher(self, rel):
		self.dict.searcher(rel)

from searcher import *
class SearcherFrame(Frame, Context, SingleVisible):
#	from view_holder import ViewHolder
	def create_widgets(self):
		rel = self.rel
		self.searcher = Searcher(self, rel['id2'](0).search_string()) #Fixme: make it static
		self.searcher.attach(self)
		self.searcher.pack(expand = 1, fill = BOTH)
		
		self.bottom_frame = Frame(self)
		self.add_button = Button(self.bottom_frame, text = "Add", underline = 0, command = self.add, state = 'disabled')
		self.add_button.pack(side = 'left')
		self.cancel_button = Button(self.bottom_frame, text = "Cancel", underline = 0, command = self.cancel)
		self.cancel_button.pack(side = 'left')
		self.bottom_frame.pack()
		from view_holder import ViewHolder
		self.view_holder = ViewHolder(self.dict, self)
		self.attach(self.view_holder)
	def __init__(self, dict, parent, rel, command):
		Frame.__init__(self, parent)
		Context.__init__(self)
		self.parent = parent
		self.dict = dict
		self.command = command
		self.rel = rel
		self.create_widgets()
	def refresh(self, message):
		ling = message.data
		if isinstance(ling, self.rel['id2']):
			self.add_button['state'] = NORMAL
			self.new_ling = ling
		else:
			self.add_button['state'] = DISABLED
		self.notify(message)
	def add(self):
		self.dict.add_rel(self.rel, self.new_ling)
	def cancel(self):
		self.command.undo()

class Editor(Frame, Context):
	def __init__(self, dict, parent):
		Frame.__init__(self, parent)
		Context.__init__(self)
		self.dict = dict
		self.search_frame = None
		self.create_widgets()
	def create_widgets(self):
		self.box = self.edit_frame = EditFrame(self.dict, self)
		self.set_box(self.edit_frame)
	def show_save_frame(self, ling):
		save_frame = SaveFrame(self.dict, self, self.dict.command(), ling.__class__)#NB
		save_frame.show()
	def show_searcher(self, rel):
		search_frame = SearcherFrame(self.dict, self, rel, self.dict.command())
		search_frame.show()
	def refresh(self, message):
##		ling = message.data
		if message.type & VM.Search:
			self.show_searcher(message.data)
		else:
			self.edit_frame.refresh(message.data)
			if message.type & VM.Edit:
				self.show_save_frame(message.data)
			elif message.type & VM.Save:
				self.box.hide()
	def set_box(self, box):
		self.box.pack_forget()
		self.box = box
		##Todo: Moje i po-dobre, prez show na SingleVisible napr.?!?!
		self.box.pack(side = BOTTOM, fill = BOTH)
		if isinstance(self.box, SearcherFrame):
			self.dict.set_view_holder(box.view_holder)
		else:
			self.dict.set_view_holder()
	def history(self):
		return self.dict.history()
	def memento(self):
		return self.box
	def set_memento(self, memento):
		self.edit_frame.refresh(self.dict.ling)
		self.set_box(memento)
	def connect_popup(self, popup):
		self.popup = popup
		popup.bind_widget(self.edit_frame)
		#popup.bind_widget(self.ling_view.box)
	
