# -*- coding: utf-8 -*-

# Copyright 2006-2012 Borislav Rizov <bobyATdclDOTbasDOTbg>, Department of Computational Linguistic, Institute for Bulgarian Language – Bulgarian Academy of Sciences
# 
# This file is part of Hydra.
# 
# Hydra is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hydra is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Hydra.  If not, see <http://www.gnu.org/licenses/>.

from wordnet import *
from xml.dom.minidom import Text, Element
import sys
import traceback

class Export:
    
    def __init__(self):
        pass
    
    def export_lang(self, lang, file):
        print >> file, '<?xml version="1.0" ?><WORDNET lang="%s">' % lang
        synsets = wn.get("lang('%s')" % lang)
#        synsets = wn.get("10007943673 | 10007910783 | 10071970213" )
#        synsets = wn.get("10071970213" )
        #print len(synsets)
        for synset in synsets:
            try:
                data = self.export_synset(synset)
                print >> file, data.toxml('utf8')
            except:
                traceback.print_exc()
        print >> file, '</WORDNET>'
    
    def tag2(self, name, value):
        return u"<%s>%s</%s>" % (name, unicode(value).rstrip(), name)
    
    def tag(self, name, *values):
        e = Element(name)
        for value in values:
            if not isinstance(value,Element):
                t = Text()
                t.data = unicode(value).rstrip()
                e.appendChild(t)
            else:
                e.appendChild(value)
        return e
    
    def export_literal(self, literal):
        word = literal["word"]
        sense = self.tag("SENSE", literal["sense"])
        lemma = self.tag("LEMMA", literal["lemma"])
        
        rels = self.export_rels(literal)
        return self.tag("LITERAL", word, sense, lemma, *rels)
        
    def export_synset(self, synset):
        ili = self.tag("ID", synset["ili"])
        pos = self.tag("POS", synset["pos"])
        defin = self.tag("DEF", synset["definition"])
        stamp = self.tag("STAMP", synset["stamp"])
        bcs = self.tag("BCS", synset["bcs"])
        frequency = self.tag("FREQUENCY",  synset["frequency"])
        domain = self.tag("DOMAIN", synset["domain"])
        sem_class = self.tag("SEM_CLASS", synset["sem_class"])
        #literals
        syns = []
        for literal in sorted(synset.literals(), key=lambda literal: literal.ID()):
            syns.append(self.export_literal(literal))
        synonyms = self.tag("SYNONYM", *syns)
        
        #ilrs, notes
        rels = self.export_rels(synset)
        
        result = self.tag("SYNSET", ili, pos, synonyms, defin, stamp, bcs, frequency, domain, sem_class, *rels)
        return result

    def export_rels(self, ling):
        rels = []
        for rel in wn.forward_relations():
            if rel['name'] == 'literal': continue
            ling_class = ling.__class__
            if rel['id1'] == ling_class:
                for l in rel.neighbours(ling):
                    exporter = getattr(self, 'format_' + l.__class__.__name__.lower())
                    rels.append(exporter(ling, rel, l))
        return rels

    def get_tag(self, ling):
        if ling.__class__ == Literal:
            return 'LR'
        return 'ILR'

    def format_note(self, ling, rel, note):
        return self.tag(rel['name'].upper(), note['note'])
    
    def format_synset(self, ling, rel, synset):
        tag = self.get_tag(ling)
        return self.tag(tag, synset['ili'], self.tag('TYPE', rel['name']))
    
    def format_literal(self, ling, rel, literal):
        tag = self.get_tag(ling)
        synset = literal.synset()
        literals = [lit.ID() for lit in synset.literals()]
        literals.sort()
        return self.tag(tag, synset['ili'] + ':' + str(literals.index(literal.ID())), self.tag('TYPE', rel['name']))


if __name__ == '__main__':
    if len(sys.argv) < 2:
        lang = 'bg'
    else:
        lang = sys.argv[1]
    export = Export()
    export.export_lang(lang, sys.stdout)
