# -*- coding: utf-8 -*-

# Copyright 2006-2012 Borislav Rizov <bobyATdclDOTbasDOTbg>, Department of Computational Linguistic, Institute for Bulgarian Language – Bulgarian Academy of Sciences
# 
# This file is part of Hydra.
# 
# Hydra is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hydra is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Hydra.  If not, see <http://www.gnu.org/licenses/>.

from Tix import *
from wordnet import *
import xml.dom.minidom
from edit_frames import *
from common import *

delim = '/'

def decode_data(data):
    sid, fid, rel = data.split(delim)
    return (long(sid), int(fid), rel)

def encode_data(sid, fid, rel):
    return `sid ` + delim + `fid` + delim + rel

class Feeder:
    def __init__(self):
        self.funcs = []
        self.frames = []

    def register_func(self, func):
        self.funcs.append(func)
    
    def refeed(self, ling, editor, path, **kwargs):
        sid, frame, rel = decode_data(editor.tree.hlist.info_data(path))
# 		sling = wn.ling(sid)
        if rel != "":
            frame = self.frames[frame](ling, editor.tree.hlist, label = rel, **kwargs)
            editor.tree.hlist.entryconfigure(path, window = frame)
        else:
            frame = self.frames[frame](ling, editor.tree.hlist, **kwargs)
            editor.tree.hlist.entryconfigure(path, window = frame)
        return frame
        
    def feed(self, ling, editor, path):
        for func in self.funcs:
            func(ling, editor, path)
    
    def set_mode(self, m):
        self.mode = m
    
    def register_frame(self, frame):
        self.frames.append(frame)
        return len(self.frames) - 1
        
class XmlConfiger:
    def value(self, node):
        val = node.childNodes[0].data
        return val
    
    def single_child(self, node, name):
        nodes = node.getElementsByTagName(name)
        for anode in nodes:
            if anode.parentNode == node:
                return anode
        return None
    
    def single(self, node, name, default = None):
        child = self.single_child(node, name)
        if child:
            return child.childNodes[0].data
        return default
    
    def __init__(self, config_name):
        self.df = 'darkblue'
        dom = xml.dom.minidom.parse(config_name)
        objs = dom.getElementsByTagName('Object')
        self.feeders = {}
        for obj in objs:
            self.process(obj)
    
    def load_size(self, obj, def_width = 100, def_height = 20):
        width = self.single(obj, "Width", def_width)
        height = self.single(obj, "Height", def_height)
        return (width, height)
    
    def load_label(self, obj, def_name = "", def_color = "black"):
        child = self.single_child(obj, "Label")
        if child:
            name = self.single(child, "Name", def_name)
            color = self.single(child, "Color", def_color)
        else:
            name = def_name
            color = def_color
        return (name, color)
    
    def create_label(self, obj):
        #               (name, color) = self.load_label(obj)
        text_color = self.single(obj, "Color", "black")
        field = self.single(obj, "Data", None)
        class MyLabel(Label):
            def __init__(self, ling, parent, labdefault = ""):
                Label.__init__(self, parent)
                self.ling = ling
                self['text'] = self.ling.to_string(field)
                self['foreground'] = text_color
        return MyLabel
    
    def create_combo(self, obj):
        (name, color) = self.load_label(obj)
#        values_table = self.single(obj, "Values")
#        vals = wn.get_values(POSId.name, values_table)
        field = self.single(obj, "Data", None)
        width, height = self.load_size(obj, 10, 1)
        df = self.df
        
        text_color = self.single(obj, "Color", "black")
        class MyCombo(ComboBox):
            def __init__(self, ling, parent):
                ComboBox.__init__(self, parent, state = 'disabled', options='listbox.height -1')
                lab = name
                self.entry['state'] = 'readonly'
                self.entry['width'] = width
                self.entry['foreground'] = text_color
                self.entry['disabledforeground'] = df
                self.label['text'] = lab
                self.ling = ling
                if ling.editable():
                    self['state'] = 'normal'
                else:
                    self['state'] = 'disabled'
                self.label['foreground'] = color
                self.var = StringVar()
                
                f_table = self.ling.descriptor.foreign[field].table
                vals = wn.pool[f_table].string_values()
                
                for val in vals:
                    self.append_history(val)
                self.var.set(ling.to_string(field))
                self['variable'] = self.var
                self['command'] = self.select
            def select(self, value):
                self.ling.from_string(value, field)
        return MyCombo
    
    def create_edit(self, obj):
        (name, color) = self.load_label(obj)
        text_color = self.single(obj, "Color", "black")
        field = self.single(obj, "Data", None)
        width, height = self.load_size(obj, 10, 1)
        df = self.df
    
        class MyEntry(LabelEntry):
            def __init__(self, ling, parent):
                LabelEntry.__init__(self, parent, label = name, state = 'readonly')
                self.entry['width'] = width
                self.label['foreground'] = color
                self.entry['foreground'] = text_color
                self.entry['disabledforeground'] = df
                self.ling = ling
                self.var = StringVar()
                #print type(ling.to_string(field))
                self.var.set(ling.to_string(field))
                self.entry['textvariable'] = self.var
                self.entry['textvariable'] = None
                if ling.editable():
                    self.entry['state'] = 'normal'
                else:
                    self.entry['state'] = 'readonly'
                self.var.trace("w", self.validate)
        #                               self.entry['validate'] = 'key'
        #                               self.entry['validatecommand'] = self.validate
            def validate(self, a, b, c):
                try:
                    self.ling.from_string(self.entry.get(), field)
                except:
                #NB can do sth
                    pass
                return True
        return MyEntry
    
    def create_text(self, obj):
        (name, color) = self.load_label(obj)
        field = self.single(obj, "Data", None)
        width, height = self.load_size(obj, 50, 3)
        df = self.df
        class MyText(Text):
            def __init__(self, ling, parent):
                Text.__init__(self, parent)
                self.ling = ling
                self['height'] = height
                self['width'] = width
                self['foreground'] = df
                self.insert('end', ling.to_string(field))
                if ling.editable():
                    self['state'] = 'normal'
                else:
                    self['state'] = 'disabled'
                self.bind('<KeyRelease>', self.validate)
            def validate(self, key):
                try:
                    self.ling.from_string(self.get('1.0', 'end'), field)
                except:
                #NB can do sth
                    pass
                return True
        return MyText
    
    def create_entry(self, obj):
        control_type = self.single(obj, "Control")
        if control_type == "label":
            control = self.create_label(obj)
        elif control_type == "option":
            control = self.create_combo(obj)
        elif control_type == "entry":
            control = self.create_edit(obj)
        elif control_type == "text":
            control = self.create_text(obj)
        else:
            raise "Unknown control"
        return control
    
    def create_frame(self, obj):
        nodes = obj.childNodes
        orientation = self.single(obj, 'Orientation', 'horizontal')
        if orientation == 'horizontal':
            side = 'left'
        else:
            side = 'top'
        (label, color) = self.load_label(obj)
        windows = []
        for node in nodes:
            if node.nodeType != node.TEXT_NODE:
                if node.tagName == 'Frame':
                    window = self.create_frame(node)
                elif node.tagName == 'Edit':
                    window = self.create_entry(node)
                else:
                    continue
                windows.append(window)
        class MyFrame(Frame):
            def __init__(self, ling, parent = None, label = label, labelcolor = color):
                Frame.__init__(self, parent)
                self.ling = ling
                self.label = Label(self)
                self.label['text'] = label
                self.label['foreground'] = labelcolor
                self.label.pack(side = 'left')
        #                               self.bind("<Button-1>", self.callback)
                for window in windows:
                    w = window(ling, self)
                    w.pack(side = side)
            def highlight(self, high):
                if high:
                    args = {'relief': 'sunken'}
                else:
                    args = {'relief': 'flat'}
                self.configure(**args)
                
        return MyFrame
    
    def create_frame_func_rel(self, obj):
        MyFrame = self.create_frame(obj)
        class RelFrame(MyFrame):
            def __init__(self, ling, rel, editor, **opt):
                base_ling = opt['base_ling']
                del opt['base_ling']
                MyFrame.__init__(self, ling, editor.tree.hlist, **opt)
                self.rel = rel
                self.ling = ling
                self.editor = editor
                if base_ling.editable() and self.rel.deletable():
                    self.del_button = Button(self, text = "delete", command = self.delete)
                    self.del_button.pack(side = 'left')
            def delete(self):
                self.editor.dict.delete_rel(self.rel, self.ling)
    
        c = self.feeder.register_frame(MyFrame)
        def func(ling, rel, editor, path, **opt):
            frame = RelFrame(ling, rel, editor, **opt)
            return editor.tree.hlist.add_child(path, itemtype = 'window', window = frame, data = encode_data(ling.ID(), c, rel['name']))
        return func
    
    def create_frame_func(self, obj):
        MyFrame = self.create_frame(obj)
        c = self.feeder.register_frame(MyFrame)
        def func(ling, editor, path, label = None):
            frame = MyFrame(ling, editor.tree.hlist)
            if(label):
                frame.label['text'] = label
            return editor.tree.hlist.add_child(path, itemtype = 'window', window = frame, data = encode_data(ling.ID(), c, ""))
        return func
    
    def create_rel(self, obj):
        frame_node = obj.getElementsByTagName("Frame")[0]
        frame_func = self.create_frame_func_rel(frame_node)
        rel = self.single(obj, "Name")
        if rel:
            relation = wn.relations[rel]
            self.rels.discard(relation)
            rels = [relation,]
        else:
            rels = self.rels
        def func(ling, editor, path):
            for r in rels:
                if ling.__class__ is r['id1']:
                    rn = r['name']
                    lings = r.neighbours(ling)
                    for aling in lings:
                        p = frame_func(aling, r, editor, path,  label = rn, base_ling = ling)
                        name = class_name(aling)
                        feeder = self.feeders[name]
                        editor.tree.setmode(p, feeder.mode)
        return func

    
    def process(self, obj):
        nodes = obj.childNodes
        name = self.single(obj, 'Name')
        feeder = Feeder()
        self.feeder = feeder
        feeder.set_mode(self.single(obj, 'Type'))
        self.rels = set(wn.relations.values())
        for node in nodes:
            if node.nodeType != node.TEXT_NODE:
                if node.tagName == 'Frame':
                    func = self.create_frame_func(node)
                elif node.tagName == 'Rel':
                    func = self.create_rel(node)
                else:
                    continue
                feeder.register_func(func)
        self.feeders[name] = feeder
        
