# -*- coding: utf-8 -*-

# Copyright 2006-2012 Borislav Rizov <bobyATdclDOTbasDOTbg>, Department of Computational Linguistic, Institute for Bulgarian Language – Bulgarian Academy of Sciences
# 
# This file is part of Hydra.
# 
# Hydra is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hydra is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Hydra.  If not, see <http://www.gnu.org/licenses/>.

from tkFileDialog import *;
import fileinput;
from dict import *;
import re;
from tkMessageBox import *

version = 0.6;

class DictHolder:
#### Menu creation ############################
    def create_menu(self):
        menu = Menu(self.top)
        self.top.config(menu=menu)
        self.filemenu = Menu(menu)
        menu.add_cascade(label="Connect", menu = self.filemenu)
    def connection(self, pos, dict, dict2, connected  = False):
        var = IntVar()
        if pos != None:
            self.filemenu.insert_checkbutton(pos, label = dict.name + "->" + dict2.name, command = lambda: dict.attach(dict2, VM.Select|VM.Open|VM.Close|VM.Save, var.get()), variable = var)
        else:
            self.filemenu.add_checkbutton(label = dict.name + "->" + dict2.name, command = lambda: dict.attach(dict2, VM.Select, var.get()), variable = var)
        var.set(connected)
        dict.attach(dict2, VM.Select|VM.Open|VM.Close|VM.Save, connected)
    def add(self, dict):
        n = len(self.dicts)
        for k, dict2 in enumerate(self.dicts):
            self.connection(None, dict, dict2)
            self.connection((k + 2)*(n), dict2, dict)
            #print (k+1)*n
        self.connection(n + 1, self.searcher, dict, True)
        dict.attach(self.searcher)
        self.dicts.append(dict)
    def remove(self, dict):
        i = self.dicts.index(dict)
        self.dicts.remove(dict)
        n = len(self.dicts)
        ii = (i + 1)*(n) + 1
        self.filemenu.delete(i + 1)
        self.filemenu.delete(ii, ii + n - 1)
        for k in reversed(range(n)):
            if k >= i:
                self.filemenu.delete(i + (k+1)*(n) + 1)
            else:
                self.filemenu.delete(i + (k+1)*n)
        
        self.searcher.attach(dict, VM.Select, 0)
        for adict in self.dicts:
            adict.attach(dict, VM.Select, 0)

#### end: Menu creation #######################
    def create_widgets(self):
        self.top = Toplevel()
        self.top.title("Hydra [" + wn.db_name() + "]")
        self.create_menu();
        self.top.protocol("WM_DELETE_WINDOW", self.on_close)
        self.top.withdraw()
        self.pane = PanedWindow(self.top, orientation = 'horizontal', width=200)
        self.pane.pack(side=TOP, padx=3, pady=3, fill=BOTH, expand=1)
    def on_close(self):
        pass
# 		print "closed"
    def closeable(self):
        for dict in self.dicts:
            if not dict.enabled():
                return False
        return True
    def __init__(self, searcher):
        self.dicts = []
        self.names = dict()
        self.searcher = searcher;
        self.create_widgets();
    def pane_name(self, dict_name):
        return "pane_%s" % dict_name
    def add_dict(self, lang):
        self.top.deiconify()
        lang_name = lang['name']
        count = self.names[lang_name] = self.names.get(lang_name, 0) + 1
        name = lang_name + `count`
        pname = self.pane_name(name)
        p = self.pane.add(pname, min = 150, size = 300)
        dict = Dict(lang, name, self, p);
        dict.pack(side = 'left', fill='both', expand = 1)
        
        self.add(dict)
        self.create_popup(dict)
        return dict
    def create_popup(self, dict):
        popup = PopupMenu(self.top, title = "Clone")
        popup.menu["postcommand"] = lambda: self.create_submenu(popup, dict)
        dict.connect_popup(popup)
        #popup.bind_widget(dict.ling_view)
        #popup.bind_widget(dict.ling_view.box)
    def create_submenu(self, popup, dict):
        menu = popup.menu
        menu.delete(0, len(self.dicts))
        for adict in self.dicts:
            if adict != dict and adict.ling:
##				print "ling for menu:", adict.ling
                menu.add_command(label = adict.name, command = lambda l = adict.ling: dict.clone(l))
    def remove_dict(self, dict):
        if dict.enabled():
            self.pane.forget(self.pane_name(dict.name))
            self.remove(dict)
            if not self.dicts:
                self.top.withdraw()	
### Main #############################################
class Main:
    def create_widgets(self):
        self.create_menu();
        # searcher
        self.searcher = Searcher(self.searcher_window)
        self.searcher.pack(expand = 1, fill = BOTH)
        # dict holder
        self.holder = DictHolder(self.searcher)

    def create_menu(self):
        menu = Menu(self.searcher_window)
        self.searcher_window.config(menu=menu)
        filemenu = Menu(menu)
        menu.add_cascade(label="File", menu=filemenu)
        filemenu.add_command(label=u'Open', command=self.open)
        for lang in wn.langs.values():
            filemenu.add_command(label=u"New %s" %lang['name'], command=self.create_mirror(lang))
        filemenu.add_command(label = u"Quit", command = self.on_close, underline = 0)

    def open(self):
        filename = askopenfilename(parent = self.searcher_window)
        if filename:
            self.searcher.set('file:%s' % filename)

    def create_mirror(self, lang):
        return lambda: self.add_dict(lang)
    def add_dict(self, lang):
        dict = self.holder.add_dict(lang)
# 		self.searcher.attach(dict)
##		self.create_popup(dict)
        
    def add_options(self, root):
        import settings
        if hasattr(settings, 'options'):
            for key, value in settings.options.iteritems():
                print key, value
                root.option_add(key, value)
        
    def __init__(self):
        self.searcher_window = Tk()
        self.add_options(self.searcher_window)
        self.searcher_window.title("Hydra: search [" + wn.db_name() + "]")
        self.searcher_window.protocol("WM_DELETE_WINDOW", self.on_close)
        self.create_widgets()
        mainloop()
        
    def on_close(self):
        if self.holder.closeable():
            if askokcancel("Close", "Are you sure?"):
                self.searcher_window.destroy()
        else:
            showinfo("Dictionaries", "Some data is not saved!")

if __name__ == '__main__':
    main = Main()
    print "That's all folks for version", version

