# -*- coding: utf-8 -*-

# Copyright 2006-2012 Borislav Rizov <bobyATdclDOTbasDOTbg>, Department of Computational Linguistic, Institute for Bulgarian Language – Bulgarian Academy of Sciences
# 
# This file is part of Hydra.
# 
# Hydra is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hydra is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Hydra.  If not, see <http://www.gnu.org/licenses/>.

from _wn import wn
from descriptor import *

class Ling:
    def __init__(self, _id, version):
        self._id = long(_id)
        self.edit_data = None
        self.version = version

    def _to_string(self, field, default = ''):
        field_value = self.data()[self.descriptor[field]]
        if field in self.descriptor.foreign:
            field_value = wn.pool[self.descriptor.foreign[field].table].to_string(field_value)
        if field_value == None:
            return default
        return field_value
    
    def to_string(self, field, default=''):
        raise 'implement me'

    def __getitem__(self, item):
        return self.to_string(item)
    
    def from_string(self, string, field):
        index = self.descriptor[field]
        if field in self.descriptor.foreign:
            f_descriptor = self.descriptor.foreign[field]
            value = wn.pool[f_descriptor.table].from_string(string)[f_descriptor.id]
        else:
            value = type(self.edit_data[index])(string)
        self.edit_data[index] = value
    
    def synset(self):
        pass
    
    def ID(self):
        return self._id
    # NB
    def editable(self):
        return self.edit_data != None
    
    def edit(self):
        self.lock()
        self.edit_data = list(self.data())
        self.version += 1
    
    def _write(self, data):
        wn.replace(self.descriptor.table, data)
    
    def save(self):
        #NB got to do it with try block
#		if self.editable()
        self._write(self.edit_data)
        self.edit_data = None
        self.unlock()
        self.version += 1
    
    def cancel(self):
        self.edit_data = None
        self.unlock()
        self.version -= 1
    
    def lang(self):
        synset = self.synset()
        if not synset.data(): print synset.ID()
        return wn.langs[synset.data()[SynsetId.lang]]
    
    def memento(self):
        return (self.edit_data, self.data())
    
    def set_memento(self, memento):
        self.edit_data = memento[0]
        self._write(memento[1])
    
    def delete(self):
        self._unlock()
        wn.execute(u"delete from %s where id = %d" % (self.descriptor.table, self.ID()))
    
    def data(self):
        return wn.get_one(self.ID(), self.descriptor.table)
    
    def locked(self):
        return wn.locked(self.ID())
    
    def _lock(self):
        #print "lock"
        wn.lock(self.ID())
    
    def _unlock(self):
        #print "unlock"
        wn.unlock(self.ID())
    
    def lock(self):
        if not self.locked():
            self._lock()
            for aling in self.neighbours():
                aling._lock()
        else:
            raise "unable to lock"
    
    def unlock(self):
        for aling in self.neighbours():
            aling._unlock()
        self._unlock()
        
    def neighbours(self):
        ns = []
        Class = self.__class__
        for rn, rel in wn.relations.iteritems():
            #if Class is rel['id2']:
                #lings = wn.get_rel(rn, self)
                #ns += lings
            if Class is rel['id1']:
                lings = rel._neighbours(self)
                ns += lings
        return ns
    
    def check(self):
        """returns error description if such presents"""
        return None
    
    @classmethod
    def create_new(self, lang):
        return self.create_new_(lang)


class Synset(Ling):
    descriptor = SynsetId
    def __init__(self, _id, version = 0):
        Ling.__init__(self, _id, version)
        self._ili = None
    
    def to_string(self, field = None, default = ''):
        if field:
            return self._to_string(field, default)
    
        lits = self.literals()
        pos = self._to_string('pos')
        return self.lang()["name"] + " - " + pos + ": " + "; ".join([lit.to_string() for lit in lits])
    
    def synset(self):
        return self
    
    @classmethod
    def create_new_(self, lang):
        ili = lang.make_ili()
        sid = wn.encode(ili, lang, 1)
        wn.insert(self.descriptor.table, (sid, ili, 0, '', '', 6, lang['id'],  0, 0, 0))
        return Synset(sid)

    # Specific		
    def literals(self):
#		return wn.get_rel("~Literal", self)
        return wn.relations['literal']._neighbours(self)
    # lazy decoding of ili
    def ili(self): 
        if not self._ili:
            self._ili = wn.decode(self)
        return self._ili
    def search_string(self):
        return "<Literal>word(%s\'%s\')"
    def cloning(self, lang):
        if lang == self.lang():
            return self #Synset(self.ID(), self.version)
        lings = wn.get("<ili>%d & lang('%s')" %(self.ID(), lang['name']))
#        print '----cloning', lings
        if lings:
            ling = lings[0]
            ling.version = self.version
            return ling
        return None
    def clone(self, lang):
        data = list(self.data())
        data[SynsetId.id] = wn.encode(data[SynsetId.ili], lang, 1)
        data[SynsetId.lang] = lang.ID()
        data[SynsetId.definition] = u""
        wn.insert(SynsetId.table, data)
        return Synset(data[SynsetId.id])
    def check(self):
        #for plain data
        if self.edit_data:
            if self.edit_data[SynsetId.definition].rstrip(" \n\r\t") == "":
                return "Missing definition!"
            if self.edit_data[SynsetId.pos] == 0:
                return "Missing pos!"
            if self.edit_data[SynsetId.bcs] < 0:
                return "Bcs can not be negative!"
        #for relations
        if len(self.literals()) == 0:
            return "Missing literal!"
##		for rel in wn.relations.values():
##			if rel["id1"] is Synset and rel["id2"] is Synset and rel.neighbours()
    def frequency(self):
#		print int(self['frequency'])
        return int(self['frequency'])
    def update_frequency(self,  value): #Todo: move into base
#        print 'update frequency',  self.ID(),  value
        wn.execute('update %s set frequency = frequency + %s where id = %s' % (SynsetId.table, value,  self.ID()))
    def create_stamp(self):
        return wn.get_user() + " " + str(wn.get_server_timestamp())
    def save(self):
        self.edit_data[SynsetId.stamp] = self.create_stamp()
        Ling.save(self)

class Literal(Ling):
    descriptor = LiteralId
    def __init__(self, _id, version = 0):
        Ling.__init__(self, _id, version)
    
    def to_string(self, field=None, default=''):
        if field:
            return self._to_string(field, default)
        data = self.data()
        return data[LiteralId.word] + ':' + `int(data[LiteralId.sense])` + ' {' + data[LiteralId.lemma] + '}'
    
    def synset(self):
        return wn.get("<Literal>%d" % self.ID())[0]
    
    def search_string(self):
        return "word(%s\'%s\')"
    
    @classmethod
    def create_new_(self, lang):
        wn.insert(self.descriptor.table, (0, 0, '', ''))
        lid = wn.last_id()
        return Literal(lid)
#    create_new_ = classmethod(create_new_)
    
    def cloning(self, lang):
        if self.lang() == lang:
            return self
        else:
            return self.synset().cloning(lang)
    
    def check(self):
        word = self.edit_data[LiteralId.word]
        if word == '':
            return "Empty literal!"
        for lit in self.synset().literals():
            if lit.ID() != self.ID() and word == lit.to_string("word"):
                return "The literal '%s' already exists!" % word
    
    def save(self):
        word = self.edit_data[LiteralId.word]
        if word != self.data()[LiteralId.word]:
            self.edit_data[LiteralId.sense] = wn.new_sense(word)
        Ling.save(self)

class Note(Ling):
    descriptor = NoteId
    def __init__(self, _id, version = 0):
        Ling.__init__(self, _id, version)
    
    def data(self):
        return wn.get_one(self.ID(), NoteId.table)
    
    def to_string(self, field=None, default=''):
        if field:
            return self._to_string(field, default)
        return self.data()[NoteId.note]
    
    def synset(self):
        form = Note.formula % {'nid': self._id}
        return wn.get(form)[0]
    
    def search_string(self):	
        #print "implement Note search_string"
        return None
    
    @classmethod
    def create_new_(self, lang):
        wn.insert(self.descriptor.table, (0, ''))
        lid = wn.last_id()
        return Note(lid)
#    create_new_ = classmethod(create_new_)
    
    def cloning(self, lang):
        if self.lang() == lang:
            return self
            #return Note(self.ID(), self.version)
        else:
            return self.synset().cloning(lang)
