# -*- coding: utf-8 -*-

# Copyright 2006-2012 Borislav Rizov <bobyATdclDOTbasDOTbg>, Department of Computational Linguistic, Institute for Bulgarian Language – Bulgarian Academy of Sciences
# 
# This file is part of Hydra.
# 
# Hydra is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hydra is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Hydra.  If not, see <http://www.gnu.org/licenses/>.

from Tix import *
from wordnet import *
from observer_context import Context, VMessage
from common import *

class Paginator:
    def __init__(self, result_set, page = 1):
        self.factor = 10
        self.result_set = result_set
        try:
            self.page = int(page)
        except:
            raise Exception('NOT A VALID PAGE NUMBER')
        if (self.page <= 0 or self.page > self.pages()) and self.pages() != 0:
            raise Exception('PAGE NUMBER OUT OF RANGE')

    def has_next(self):
        return self.page < self.pages()

    def has_previous(self):
        return self.page > 1

    def get_page(self):
        l = (self.page-1) * self.factor
        r = min((self.page) * self.factor,  self.all())
        return [ (i,self.result_set[i]) for i in range(l,  r)]

    def next(self):
        if self.has_next():
            self.page += 1

    def previous(self):
        if self.has_previous():
            self.page -= 1

    def pages(self):
        return (len(self.result_set) + self.factor - 1)/ self.factor

    def all(self):
        return len(self.result_set)

class Searcher(Frame, Context):
    max_res = 100
    def __init__(self,parent = None, formula = "<Literal>word(%s\'%s\')"):
        self.name = "searcher"
        Frame.__init__(self, parent)
        Context.__init__(self)
        self.formula = formula
        self.create_widgets()
        self.paginator = Paginator([])
        self.lings = []
    def nav_prev(self):
        self.paginator.previous()
        self.show_results()
    def nav_next(self):
        self.paginator.next()
        self.show_results()
   
    def create_paginator(self):
        frame = Frame(self)
        self.nav_label = Label(frame, text = '')
        self.nav_label.pack(side = 'left')
        self.prev_button = Button(frame, text = '<', command = self.nav_prev,  state = 'disabled')
        self.next_button = Button(frame, text = '>', command = self.nav_next,  state = 'disabled')
        self.next_button.pack(side = 'right')
        self.prev_button.pack(side = 'right')
        return frame

    def create_widgets(self):
        string_input = ComboBox(self, label = "input:", command = self.search, editable = 1, history = 1, options='listbox.height -1 label.anchor e')
        string_input.pack(fill='x')
        self.string_input = string_input
        
        frame = Frame(self)
        label_formula = Label(frame, text = "formula")
        label_formula.pack(side = 'left')
        self.is_formula = BooleanVar(False)
        self.is_rex = BooleanVar(False)
        self.is_split = BooleanVar(False)
        self.check_formula = Checkbutton(frame, variable = self.is_formula, command = self.formula_controls)
        self.check_formula.pack(side = 'left')
        
        
        self.label_rex = Label(frame, text = "rex")
        self.label_rex.pack(side = 'left')

        self.check_rex = Checkbutton(frame, variable = self.is_rex)
        self.check_rex.pack(side = 'left')

        self.check_split = Checkbutton(frame, variable = self.is_split)
#        self.check_split.pack(side = 'right')
        
        self.label_split = Label(frame, text = "split")
#        self.label_split.pack(side = 'left')

        frame.pack(anchor = 'w')
        
        self.result_list = ScrolledListBox(self, browsecmd = self.browse, command = self.print_current)
        self.curselection = -1
        self.lb = self.result_list.listbox
        self.lb['selectmode'] = None
        self.lb.configure(bg = 'ivory')
        self.result_list.pack(fill = 'both', expand = 1)
        paginator = self.create_paginator()
        paginator.pack(fill = 'x',  anchor = 's')

    def set(self, value):
        self.string_input.add_history(value)
#        size = self.string_input.slistbox.listbox.size()
#        self.string_input.slistbox.listbox.configure(height=min(10, size))
        self.string_input.pick(0)

    def set_state(self,  state):
        self.label_rex['state'] = state
        self.check_rex['state'] = state
        
        self.label_split['state'] = state
        self.check_split['state'] = state

    def formula_controls(self):
        if self.is_formula.get():
            self.set_state('disabled')
        else:
            self.set_state('normal')

    def check(self,  lings):
        result = []
        for ling in lings:
            if ling.data():
                result.append(ling)
        return result
    
    def get(self,  formula):
        return self.check(wn.get(formula))
    
    def load(self, filename):
        lines = open(filename, 'rt').readlines()
        return [wn.ling(long(id.strip().split(' ')[0])) for id in lines if len(id) > 1]
    
    def search(self, string):
        x = string
        if string.startswith('file:'):
            self.lings = self.load(string[5:])
        elif self.is_formula.get():
            self.lings =  self.get(x)
        elif self.is_split.get():
            self.lings = self.get('%s|<ili>%s|<ili><~split>%s' % (x,  x,  x))
        else:
            if self.is_rex.get():
                r = '#'
            else:
                r = ''
            query = self.formula % (r, x)
            self.lings =  self.get(query)
        self.paginator = Paginator(self.lings)
        self.show_results()
    
    def _state(self,  control,  state):
        if state:
            control['state'] = 'normal'
        else:
            control['state'] = 'disabled'
    
    def show_results(self):
        self.lb.delete(0, END) # clear
        for i,  ling in self.paginator.get_page():
            self.lb.insert(END, class_name(ling) + ": " + ling.to_string())
        self.nav_label['text'] = ' %s results, page %s of %s.' % (self.paginator.all(),  self.paginator.page,  self.paginator.pages())
        self._state(self.prev_button,  self.paginator.has_previous())
        self._state(self.next_button,  self.paginator.has_next())

    def refresh(self, message):
        aling = message.data
        if aling:
            for i, ling in self.paginator.get_page():
                if ling.ID() == aling.ID():
                    self.lb.delete(i)
                    self.lb.insert(i,  class_name(ling) + ": " + ling.to_string())

    def print_current(self):
        sel = self.lb.curselection() 
        if sel:
            curselection = long(sel[0]) + self.paginator.factor * (self.paginator.page - 1)
            self.notify(VMessage(self.lings[curselection]))

    def browse(self):
        sel = self.lb.curselection()
        if sel and sel[0] != self.curselection:
            self.print_current()
            self.curselection = sel[0]
