# -*- coding: utf-8 -*-

# Copyright 2006-2012 Borislav Rizov <bobyATdclDOTbasDOTbg>, Department of Computational Linguistic, Institute for Bulgarian Language – Bulgarian Academy of Sciences
#
# This file is part of Hydra.
#
# Hydra is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Hydra is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Hydra.  If not, see <http://www.gnu.org/licenses/>.

from command import *
from linguistic_units import *
class EditLingCommand(Command):
    def __init__(self, h, ling):
        Command.__init__(self, h)
        self.ling = ling
    def execute(self):
        self.ling.edit()
    def unexecute(self):
        self.ling.cancel()
    def possible_redo(self):
        return not (self.ling and self.ling.locked())

class SaveLingCommand(Command):
    def __init__(self, h, ling):
        Command.__init__(self, h)
        self.ling = ling
        self.memento = None
    def execute(self):
        ling = self.ling
        if self.memento:
            ling.set_memento(self.memento)
        else:
            self.memento = ling.memento()
        ling.save()
    def unexecute(self):
        self.ling.set_memento(self.memento)
    def possible_undo(self):
        return not self.ling.locked()

class RefreshCommand(Command):
    def __init__(self, h, dict, message):
        Command.__init__(self, h)
        ling = message.data
        self.dict = dict
        self.ling = ling
        self.message = message.copy()
    def execute(self):
        self.memento = self.dict.memento()
        self.dict._refresh(self.message.copy())
    def unexecute(self):
        self.dict.set_memento(self.memento)
class AddLingCommand(Command):
    def __init__(self, h, LingClass, lang):
        Command.__init__(self, h)
        self.LingClass = LingClass
        self.lang = lang
        self.ling = None
    def execute(self):
        if not self.ling:
            self.ling = self.LingClass.create_new(self.lang)
        else:
            self.ling.set_memento(self.memento)
        self.memento = self.ling.memento()
    def unexecute(self):
        self.ling.delete()
    def possible_undo(self):
        return not self.ling.locked()
class AddCreationalRelCommand(Command):
    def __init__(self, h, ling, rel):
        Command.__init__(self, h)
        self.ling = ling
        self.rel = rel
        self.add_ling = AddLingCommand(h, rel['id2'], self.ling.lang())
    def execute(self):
        self.add_ling.execute()
        self.add_rel = AddRelCommand(self.history, self.rel, self.ling, self.add_ling.ling)
        self.add_rel.execute()
    def new_ling(self):
        return self.add_ling.ling
    def unexecute(self):
        self.add_rel.unexecute()
        self.add_ling.unexecute()
class NewCreationalLingCommand(Command):
    def __init__(self, h, dict, sling, rel):
        Command.__init__(self, h)
        self.sling = sling
        self.dict = dict
        self.rel = rel
        self.add_creational_rel_command = AddCreationalRelCommand(h, self.sling, self.rel)
##		self.edit_inner_ling_command = EditInnerLingCommand(h, self.dict, self.sling, ling)
    def execute(self):
        h = self.history
        self.add = MacroCommand2(h)
        self.add.append(self.add_creational_rel_command)
        self.add.append(RefreshCommand(h, self.dict, VMessage(self.sling)))
        self.add.execute()

        ling = self.add_creational_rel_command.new_ling()
        self.edit_inner_ling_command = EditInnerLingCommand(h, self.dict, self.sling, ling)
        self.edit_inner_ling_command.execute()
    def unexecute(self):
        self.edit_inner_ling_command.unexecute()
        self.add.unexecute()

class NewSynsetCommand(Command):
    def __init__(self, h, lang, dict):
        Command.__init__(self, h)
        self.lang = lang
        self.dict = dict
        self.add_ling = AddLingCommand(h, Synset, self.lang)
    def execute(self):
        h = self.history
        self.add_ling.execute()

        self.edit_ling = EditLingCommand(h, self.add_ling.ling)
        self.edit_ling.execute()

        self.refresh = RefreshCommand(h, self.dict, VMessage(self.add_ling.ling, VM.Edit|VM.Select))
        self.refresh.execute()
    def unexecute(self):
        self.edit_ling.unexecute()
        self.add_ling.unexecute()
        self.refresh.unexecute()
#end edited
class CloneSynsetLingCommand(Command):
    def __init__(self, h, ling, lang):
        Command.__init__(self, h)
        self.lang = lang
        self.pro_ling = ling
        self.ling = None
    def execute(self):
        ling = self.pro_ling.clone(self.lang)
        if self.ling:
            self.ling.set_memento(ling.memento())
        else:
            self.ling = ling
    def unexecute(self):
        self.ling.delete()

#Fix like NewSynsetCommand
class CloneSynsetCommand(MacroCommand):
    #Todo: possible_redo-check if self.ling exists.
    def __init__(self, h, ling, lang):
        MacroCommand.__init__(self, h)
        self.lang = lang
        self.ling = ling
        self.add_ling_command = CloneSynsetLingCommand(self.history, self.ling, self.lang)
    def execute(self):
        self.clear()
        self.add_ling_command.execute()
        self.append(self.add_ling_command)

        self.new_ling = new_ling = self.add_ling_command.ling
        for rn, rel in wn.sys_relations.iteritems():
            add_rel_command = AddRelCommand(self.history, rel, new_ling, new_ling)
            add_rel_command.execute()
            self.append(add_rel_command)

        for rn, rel in wn.relations.iteritems():
            if rel["id2"] is Synset:
##				lings = wn.get_rel("~" + rn, self.ling)
                lings = rel.neighbours(self.ling)
                for l in lings:
                    ling = l.cloning(self.lang)
                    if ling:
                        add_rel_command = AddRelCommand(self.history, rel, new_ling, ling)
                        add_rel_command.execute()
                        self.append(add_rel_command)
class Clone(Command):
    def __init__(self, h, dict, ling, lang):
        Command.__init__(self, h)
        self.ling = ling
        self.lang = lang
        self.dict = dict
        self.clone_synset_command = CloneSynsetCommand(h, self.ling, self.lang)
    def execute(self):
        h = self.history
        self.clone_synset_command.execute()

        new_ling = self.clone_synset_command.new_ling

        self.edit_ling_command = EditLingCommand(h, new_ling)
        self.edit_ling_command.execute()

        self.refresh_command = RefreshCommand(h, self.dict, VMessage(new_ling, VM.Edit|VM.Select))
        self.refresh_command.execute()
    def unexecute(self):
        self.edit_ling_command.unexecute()
        self.clone_synset_command.unexecute()
        self.refresh_command.unexecute()

class DeleteLingAndRelationsCommand(MacroCommand):
    def __init__(self, h, ling):
        #print h
        MacroCommand.__init__(self, h)
        self.ling = ling
        for rel in wn.forward_relations():
            rn = rel["name"]
            if rel["id1"] == ling.__class__:
                lings = wn.get_rel("~" + rn, self.ling)
                for l in lings:
                    self.append(DeleteRelCommand(h, rel, self.ling, l))
            if rel["id2"] == ling.__class__:
                lings = wn.get_rel(rn, self.ling)
                for l in lings:
                    self.append(DeleteRelCommand(h, rel, l, self.ling))

        for rel in wn.sys_relations.values():
            if rel["id1"] == ling.__class__:
                self.append(DeleteRelCommand(h, rel, self.ling, self.ling))
        self.append(DeleteLingCommand(h, self.ling))


class DeleteSynsetCommand(DeleteLingAndRelationsCommand):
    pass

class DeleteLingCommand(Command):
    def __init__(self, h, ling):
        Command.__init__(self, h)
        #print h
        self.ling = ling
    def execute(self):
        #print "delete ling"
        self.memento = self.ling.memento()
        self.ling.delete()
    def unexecute(self):
        #print "undelete ling"
        self.ling.set_memento(self.memento)
    def possible_redo(self):
        return not self.ling.locked()


class AddRelCommand(Command):
    def __init__(self, h, rel, l1, l2):
        Command.__init__(self, h)
        self.rel = rel
        self.l1 = l1
        self.l2 = l2
    def execute(self):
        self.rel.add(self.l1, self.l2)
    def unexecute(self):
        self.rel.delete(self.l1, self.l2)


class DeleteRelCommand(Command):
    def __init__(self, h, rel, l1, l2):
        Command.__init__(self, h)
        self.rel = rel
        self.l1 = l1
        self.l2 = l2
    def execute(self):
        self.rel.delete(self.l1, self.l2)
    def unexecute(self):
        self.rel.add(self.l1, self.l2)


class AddRelNewLingCommand(Command):
    def __init__(self, h, rel, ling, storage):
        Command.__init__(self, h)
        self.rel = rel
        self.ling = ling
        self.storage = storage
        self.add_ling = AddLingCommand(h, rel['id2'], ling.lang(), storage)

    def execute(self):
        self.add_ling.execute()
        self.add_rel = AddRelCommand(self.history, self.rel, self.ling, self.storage.get())
        self.add_rel.execute()
        #LingClass = self.rel['id2']
        #self.new_ling = LingClass.create_new(self.ling.lang())
        #self.storage.set(self.new_ling)
        ##print self.new_ling, "added", self
        #self.rel.add(self.ling, self.new_ling)
    def unexecute(self):
        #self.rel.delete(self.ling, self.new_ling)
        self.add_rel.unexecute()
        self.add_ling.unexecute()
        #self.new_ling.delete()
class EditInnerLingCommand(MacroCommand):
    def __init__(self, h, dict, sling, ling):
        MacroCommand.__init__(self, h)
        self.ling = ling
        self.sling = sling
        save_current_command = MacroCommand2(h)
        save_current_command.append(SaveLingCommand(h, sling))
        save_current_command.append(RefreshCommand(h, dict, VMessage(sling)))

        edit_child_command = MacroCommand2(h)
        edit_child_command.append(EditLingCommand(h, ling))
        edit_child_command.append(RefreshCommand(h, dict, VMessage(ling, VM.Edit|VM.Select)))

        self.append(save_current_command)
        self.append(edit_child_command)
